@@ -1,6 +1,9 @@
 package com.ozacc.mail;
 
+import java.io.File;
+import java.io.InputStream;
 import java.io.UnsupportedEncodingException;
+import java.net.URL;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.HashMap;
@@ -8,9 +11,14 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 
+import javax.activation.DataSource;
+import javax.activation.FileDataSource;
+import javax.activation.FileTypeMap;
+import javax.activation.URLDataSource;
 import javax.mail.internet.AddressException;
 import javax.mail.internet.InternetAddress;
 
+import com.ozacc.mail.impl.ByteArrayDataSource;
 import com.ozacc.mail.impl.Cp932;
 
 /**
@@ -18,7 +26,7 @@ import com.ozacc.mail.impl.Cp932;
  * 
  * @since 1.0
  * @author Tomohiro Otsuka
- * @version $Id: Mail.java,v 1.7 2004/09/14 00:04:57 otsuka Exp $
+ * @version $Id: Mail.java,v 1.11 2005/01/17 09:08:16 otsuka Exp $
  */
 public class Mail {
 
@@ -51,6 +59,10 @@ public class Mail {
 
 	private Map xHeaders;
 
+	private String htmlText;
+
+	private List attachmentFiles;
+
 	/**
 	 * 󥹥ȥ饯
 	 */
@@ -72,6 +84,7 @@ public class Mail {
 
 	/**
 	 * ԡ󥹥ȥ饯
+	 * ԡ(shallow copy)Ǥ
 	 * 
 	 * @since 1.0.2
 	 * 
@@ -89,6 +102,8 @@ public class Mail {
 		this.text = original.text;
 		this.to = original.to;
 		this.xHeaders = original.xHeaders;
+		this.htmlText = original.htmlText;
+		this.attachmentFiles = original.attachmentFiles;
 	}
 
 	/**
@@ -487,6 +502,7 @@ public class Mail {
 		buf.append("To: ").append(arrayToCommaDelimitedString(to)).append("\n");
 		buf.append("Cc: ").append(arrayToCommaDelimitedString(cc)).append("\n");
 		buf.append("Bcc: ").append(arrayToCommaDelimitedString(bcc)).append("\n");
+		buf.append("Reply-To: ").append(replyTo != null ? replyTo.toUnicodeString() : null).append("\n");
 		buf.append("Subject: ").append(subject).append("\n");
 
 		if (xHeaders != null) {
@@ -500,6 +516,11 @@ public class Mail {
 		buf.append("\n");
 		buf.append(text);
 
+		if (htmlText != null) {
+			buf.append("\n\n-----\n\n");
+			buf.append(htmlText);
+		}
+
 		return buf.toString();
 	}
 
@@ -549,11 +570,268 @@ public class Mail {
 		bcc = null;
 	}
 
+	/**
+	 * HTMLʸ򥻥åȤޤ
+	 * 
+	 * @since 1.1
+	 * 
+	 * @param htmlText HTMLʸ
+	 */
+	public void setHtmlText(String htmlText) {
+		this.htmlText = htmlText;
+	}
+
+	/**
+	 * HTMLʸ֤ޤ
+	 * 
+	 * @since 1.1
+	 * 
+	 * @return HTMLʸޤnull
+	 */
+	public String getHtmlText() {
+		return htmlText;
+	}
+
+	/**
+	 * ꤵ줿եźդޤ
+	 * źեե̾ˤϡꤵ줿ե̾Ѥޤ
+	 * Υե̾ŬڤʳĥҤդƤɬפޤ
+	 * 
+	 * @since 1.1
+	 * 
+	 * @param file źեե
+	 */
+	public void addFile(File file) {
+		if (attachmentFiles == null) {
+			initAttachmentFiles();
+		}
+		addFile(file, file.getName());
+	}
+
+	/**
+	 * ꤵ줿եźդޤ
+	 * ꤹե̾ˤŬڤʳĥҤդƤɬפޤ
+	 * 
+	 * @since 1.1
+	 * 
+	 * @param file źեե
+	 * @param fileName ե̾
+	 */
+	public void addFile(File file, String fileName) {
+		if (attachmentFiles == null) {
+			initAttachmentFiles();
+		}
+		attachmentFiles.add(new AttachmentFile(fileName, file));
+	}
+
+	/**
+	 * ꤵ줿URLΥեźդޤ
+	 * ꤹե̾ˤŬڤʳĥҤդƤɬפޤ
+	 * 
+	 * @since 1.1
+	 * 
+	 * @param url źեե
+	 * @param fileName ե̾
+	 */
+	public void addFile(URL url, String fileName) {
+		if (attachmentFiles == null) {
+			initAttachmentFiles();
+		}
+		attachmentFiles.add(new AttachmentFile(fileName, url));
+	}
+
+	/**
+	 * ꤵ줿InputStreamեȤźդޤ
+	 * ꤹե̾ˤŬڤʳĥҤդƤɬפޤ
+	 * 
+	 * @since 1.1
+	 * 
+	 * @param is źեեInputStream
+	 * @param fileName ե̾
+	 */
+	public void addFile(InputStream is, String fileName) {
+		if (attachmentFiles == null) {
+			initAttachmentFiles();
+		}
+		attachmentFiles.add(new AttachmentFile(fileName, is));
+	}
+
+	/**
+	 * attachmentFilesץѥƥ
+	 */
+	private void initAttachmentFiles() {
+		attachmentFiles = new ArrayList();
+	}
+
+	/**
+	 * źեե֤ޤ
+	 * źեե뤬åȤƤʤϡ֤ޤ
+	 * 
+	 * @since 1.1
+	 * 
+	 * @return źեե󡣤ޤ϶
+	 */
+	public AttachmentFile[] getAttachmentFiles() {
+		if (attachmentFiles == null) {
+			return new AttachmentFile[0];
+		}
+		return (AttachmentFile[])attachmentFiles
+				.toArray(new AttachmentFile[attachmentFiles.size()]);
+	}
+
+	/**
+	 * HTMLʸåȤƤ뤫ɤȽꤷޤ
+	 * 
+	 * @since 1.1
+	 * 
+	 * @return HTMLʸåȤƤ true
+	 */
+	public boolean isHtmlMail() {
+		return (htmlText != null);
+	}
+
+	/**
+	 * ե뤬źդƤ뤫ɤȽꤷޤ
+	 * 
+	 * @since 1.1
+	 * 
+	 * @return ե뤬źդƤ true
+	 */
+	public boolean isFileAttached() {
+		return attachmentFiles != null && attachmentFiles.size() > 0;
+	}
+
+	/**
+	 * ޥѡȡ᡼뤫ɤȽꤷޤ<br>
+	 * HTMLʸåȤƤ뤫ե뤬źդƤ true ֤ޤ
+	 * <p>
+	 * : ȽꤵޥѡȤϡ̩ʰ̣ǤΥޥѡȤǤϤޤ
+	 * 
+	 * @since 1.1
+	 * 
+	 * @return ޥѡȡ᡼ξ true
+	 */
+	public boolean isMultipartMail() {
+		return isHtmlMail() || isFileAttached();
+	}
+
+	/**
+	 * åȤƤźեեƥꥢޤ
+	 * 
+	 * @since 1.1
+	 */
+	public void clearFile() {
+		initAttachmentFiles();
+	}
+
+	/**
+	 * źեե롣
+	 * 
+	 * @since 1.1
+	 * @author Tomohiro Otsuka
+	 * @version $Id: Mail.java,v 1.11 2005/01/17 09:08:16 otsuka Exp $
+	 */
+	public class AttachmentFile {
+
+		private String name;
+
+		private File file;
+
+		private InputStream is;
+
+		private URL url;
+
+		/**
+		 * ե̾ȥեꤷơΥ饹Υ󥿥󥹤ޤ
+		 * ե̾ˤŬڤʳĥҤդƤɬפޤ
+		 * 
+		 * @param name ᡼ɽե̾
+		 * @param file źեե
+		 */
+		public AttachmentFile(String name, File file) {
+			this.name = name;
+			this.file = file;
+		}
+
+		/**
+		 * ե̾InputStreamꤷơΥ饹Υ󥿥󥹤ޤ
+		 * ե̾ˤŬڤʳĥҤդƤɬפޤ
+		 * 
+		 * @param name ᡼ɽե̾
+		 * @param is źեեInputStream
+		 */
+		public AttachmentFile(String name, InputStream is) {
+			this.name = name;
+			this.is = is;
+		}
+
+		/**
+		 * ե̾ȥեURLꤷơΥ饹Υ󥿥󥹤ޤ
+		 * ե̾ˤŬڤʳĥҤդƤɬפޤ
+		 * 
+		 * @param name ᡼ɽե̾
+		 * @param url źեեΥURL
+		 */
+		public AttachmentFile(String name, URL url) {
+			this.name = name;
+			this.url = url;
+		}
+
+		/**
+		 * źեեDataSource󥹥󥹤֤ޤ
+		 * 
+		 * @return źեեDataSource󥹥
+		 */
+		public DataSource getDataSource() {
+			if (file != null) {
+				return new FileDataSource(file);
+			}
+
+			if (url != null) {
+				return new URLDataSource(url);
+			}
+
+			// InputStreamDataSource
+			String contentType = FileTypeMap.getDefaultFileTypeMap().getContentType(name);
+			return new ByteArrayDataSource(is, contentType);
+		}
+
+		/**
+		 * źեե֤̾ޤ
+		 * 
+		 * @return źեե̾
+		 */
+		public String getName() {
+			return name;
+		}
+
+		/**
+		 * @return åȤ줿ե롣ޤnull
+		 */
+		public File getFile() {
+			return file;
+		}
+
+		/**
+		 * @return åȤ줿InputStreamޤnull
+		 */
+		public InputStream getInputStream() {
+			return is;
+		}
+
+		/**
+		 * @return åȤ줿URLޤnull
+		 */
+		public URL getUrl() {
+			return url;
+		}
+	}
+
 	/**
 	 * ᡼ν١Τߤ
 	 * 
 	 * @author Tomohiro Otsuka
-	 * @version $Id: Mail.java,v 1.7 2004/09/14 00:04:57 otsuka Exp $
+	 * @version $Id: Mail.java,v 1.11 2005/01/17 09:08:16 otsuka Exp $
 	 */
 	public static class Importance {
 
